import pandas as pd
import plotly.express as px
import sqlalchemy

from gandai.db import connect_with_connector

db = connect_with_connector()


def weekly_cumulative_events(event_type="validate") -> pd.DataFrame:
    statement = """
    SELECT 
        e.*, 
        to_timestamp(e.created) as dt,  
        EXTRACT(DOW FROM TO_TIMESTAMP(e.created)) as dow,
        a.name, 
        s.label,
        COUNT(*) OVER (PARTITION BY e.actor_key ORDER BY date_trunc('minute', to_timestamp(e.created))) as cumulative_events
    FROM event e
    JOIN actor a on e.actor_key = a.key
    JOIN search s on e.search_uid = s.uid
    WHERE 
        to_timestamp(e.created) >= date_trunc('week', current_date)
        AND a.key not in ('grata','dealcloud','7138248581')
        AND e.type = :event_type
    ORDER BY e.actor_key, date_trunc('minute', to_timestamp(e.created))
    """
    with db.connect() as conn:
        result = conn.execute(sqlalchemy.text(statement), {"event_type": event_type})
        df = pd.DataFrame(result.fetchall(), columns=result.keys())
    return df


def draw_weekly_cumulative(event_type: str) -> px.line:
    df = weekly_cumulative_events(event_type=event_type)
    return px.line(
        df,
        x="dt",
        y="cumulative_events",
        color="name",
        title=f"{event_type}",
    )


def draw_validation_per_day() -> pd.DataFrame:
    statement = """
    SELECT e.*, to_timestamp(e.created) as dt, a.name, s.label
    FROM event e
    JOIN actor a on e.actor_key = a.key
    JOIN search s on e.search_uid = s.uid
    WHERE to_timestamp(e.created) > now() - interval '7 day'
    and a.key not in ('grata','dealcloud')
    and e.type in ('validate')
    ORDER BY created
    """
    with db.connect() as conn:
        result = conn.execute(sqlalchemy.text(statement))
        df = pd.DataFrame(result.fetchall(), columns=result.keys())

    df["date"] = df["dt"].dt.strftime("%Y-%m-%d")

    validations = (
        df.groupby(["name", "date", "label"])
        .size()
        .reset_index(name="count")
        .sort_values(by=["date"])
        .reset_index(drop=True)
    )

    fig = px.bar(
        validations,
        x="date",
        y="count",
        color="name",
        barmode="group",
        title="Validations per day by search by researcher | Trailing 7 days",
        hover_data=["label"],
    )
    return fig



import time
from concurrent.futures import ThreadPoolExecutor
from functools import partial
from typing import List

import googlemaps
import pandas as pd
import requests

from gandai import gpt, helpers, models, query, secrets, constants


gmaps = googlemaps.Client(key=secrets.access_secret_version("GOOLE_MAPS_KEY"))
MAX_WORKERS = 25


class GoogleMapsWrapper:
    @staticmethod
    def get_loc(text: str = "San Diego, CA") -> tuple:
        resp = gmaps.geocode(text)
        return tuple(resp[0]["geometry"]["location"].values())

    @staticmethod
    def enrich(place_id: str) -> dict:
        resp = gmaps.place(place_id=place_id)
        return resp["result"]

    @staticmethod
    def fetch_unique_place_ids(
        search_phrase: str, locations: List[str], radius_miles: int = 25
    ) -> list:
        main_func = partial(
            GoogleMapsWrapper._fetch_place_ids,
            search_phrase=search_phrase,
            radius_miles=radius_miles,
        )
        with ThreadPoolExecutor(max_workers=MAX_WORKERS) as exec:
            futures = exec.map(main_func, locations)

        place_ids = []
        for future in futures:
            try:
                place_ids.extend(list(future))
            except Exception as e:
                print(e)

        place_ids = list(set(place_ids))
        len(place_ids)
        return place_ids

    @staticmethod
    def _fetch_place_ids(
        location_text: str, search_phrase: str, radius_miles: int = 25
    ) -> list:
        try:
            METERS_PER_MILE = 1609.34
            radius_meters = radius_miles * METERS_PER_MILE

            loc: tuple = GoogleMapsWrapper.get_loc(location_text)

            results = []

            response = gmaps.places(
                query=search_phrase,
                location=loc,
                radius=radius_meters,
            )

            results.extend(response["results"])
            next_page_token = response.get("next_page_token", None)
            while next_page_token:
                time.sleep(2)
                response = gmaps.places(
                    query=search_phrase,
                    location=loc,
                    radius=radius_meters,
                    page_token=next_page_token,
                )
                results.extend(response["results"])
                next_page_token = response.get("next_page_token", None)

            place_ids = [result["place_id"] for result in results]
            print(
                f"{search_phrase} in {location_text} within {radius_miles} miles -> {len(place_ids)} results"
            )
            return place_ids
        except Exception as e:
            print(f"Error with {location_text} {search_phrase}: {e}")
            return []

    @staticmethod
    def build_target_from_place_id(
        place_id: str, search_uid: int, append_to_prompt: str = None
    ) -> models.Company:
        """takes in an (eriched) place and inserts it as a company"""
        # really more of an upsert, but multiple txns for now

        place = GoogleMapsWrapper.enrich(place_id)

        existing_search_domains = query.unique_domains(search_uid=search_uid)[
            "domain"
        ].to_list()

        if "website" not in place:
            print(f"no website for {place_id}")
            return

        company = models.Company(
            name=place["name"],
            domain=helpers.clean_domain(place["website"]),
            description="",
        )
        print(f"starting to add {company.domain} - search_uid {search_uid}")
        
        query.insert_company(company)  
        # save the place in meta
        company = query.find_company_by_domain(company.domain)
        if "google_places" not in company.meta:
            company.meta["google_places"] = {}
        company.meta["google_places"][place_id] = place
        query.update_company(company)
        ###

        # actually this should cache the result on the company....
        if helpers.clean_domain(place["website"]) in existing_search_domains:
            print(f"skipping {place['website']} - already in search_uid {search_uid}")
            return

        # otherwise add the company
        query.insert_event(
            models.Event(
                search_uid=search_uid,
                actor_key="google",
                type="create",
                domain=company.domain,
            )
        )

        # and add the comment
        gpt_prompt = (" ").join(
            [
                f"You are doing important reasearch and should only answer if you have high confidence.",
                f"Based on these reviews:",
                f"{[review['text'] for review in place['reviews']]}",
                f"as well as what you already know about {place['website']},",
                f"what products and services does {place['name']} offer?",
            ]
        )

        if append_to_prompt:
            gpt_prompt += " " + append_to_prompt

        
        query.insert_event(
            models.Event(
                search_uid=search_uid,
                actor_key="chatgpt",
                type="comment",
                domain=company.domain,
                data={"comment": gpt.ask_gpt(gpt_prompt, max_tokens=100)},
            )
        )

        return company

    # def build_place_ids_async(place_ids: list, search: models.Search):
    #     with ThreadPoolExecutor(max_workers=MAX_WORKERS) as executor:
    #         for place_id in place_ids:
    #             executor.submit(
    #                 GoogleMapsWrapper.build_target_from_place_id,
    #                 place_id=place_id,
    #                 search_uid=search.uid,
    #             )


class GrataWrapper:
    HEADERS = {
        "Authorization": secrets.access_secret_version("GRATA_API_TOKEN"),
        "Content-Type": "application/json",
    }

    def find_similar(domain: str, search: models.Search) -> list:
        api_filters = GrataWrapper._get_api_filter(search)
        similiar_filters = {
            "domain": domain,
            "grata_employees_estimates_range": api_filters[
                "grata_employees_estimates_range"
            ],
            "headquarters": api_filters["headquarters"],
            "ownership": api_filters["ownership"],
            "exclude": api_filters["exclude"],
        }
        # print(similiar_filters)
        response = requests.post(
            "https://search.grata.com/api/v1.2/search-similar/",
            headers=GrataWrapper.HEADERS,
            json=similiar_filters,
        )
        data = response.json()
        # print("find_similar:", data)
        data["companies"] = data.get("results", [])  # asking grata about this

        return data["companies"]


    def find_by_criteria(search: models.Search) -> list:
        pages = search.criteria.result_count / 25
        companies = []
        api_filters = GrataWrapper._get_api_filter(search)
        def _find_by_criteria(api_filters: dict, page_token=None) -> dict:
            if page_token:
                api_filters["page_token"] = page_token
            response = requests.post(
                "https://search.grata.com/api/v1.2/search/",
                headers=GrataWrapper.HEADERS,
                json=api_filters,
            )
            data = response.json()
            return data
        print(api_filters)
        resp = _find_by_criteria(api_filters)
        page_token = resp.get("page_token")
        companies.extend(resp.get("companies", []))
        if pages > 1:
            for _ in range(1, int(pages)):
                resp = _find_by_criteria(api_filters, page_token)
                companies.extend(resp.get("companies", []))
                page_token = resp.get("page_token")
                if not page_token:
                    break
        
        # should I return last page token?
        return companies

    def enrich(domain: str) -> dict:
        response = requests.post(
            "https://search.grata.com/api/v1.2/enrich/",
            headers=GrataWrapper.HEADERS,
            json={"domain": domain},
        )
        data = response.json()
        data["linkedin"] = data.get("social_linkedin")
        data["ownership"] = data.get("ownership_status")
        return data

    def enrich_targets_async(targets: pd.DataFrame) -> None:
        def enrich_company_by_domain(domain: str) -> None:
            company = query.find_company_by_domain(domain)
            if "company_uid" not in company.meta.keys():
                # print(company.name)
                resp = GrataWrapper.enrich(domain)
                company.description = resp.get("description")
                company.meta = {**company.meta, **resp}  # merge 3.5+
                query.update_company(company)

            with ThreadPoolExecutor(max_workers=5) as executor:
                executor.map(enrich_company_by_domain, targets["domain"].tolist())

    def _get_api_filter(search: models.Search) -> dict:

        def _hq_include() -> list:
            hq_include = []
            cities = search.criteria.inclusion.city
            states = search.criteria.inclusion.state
            countries = search.criteria.inclusion.country

            if len(cities) > 0:
                # front-end validates only one state when city selected
                state = constants.STATES[states[0]]
                for city in cities:
                    hq_include.append(
                        {"city": city, "state": state, "country": "United States"}
                    )
                return hq_include

            if len(states) > 0:
                for state in states:
                    hq_include.append({"state": constants.STATES[state]})
            elif len(countries) > 0:
                for country in countries:
                    hq_include.append({"country": constants.COUNTRIES[country]})
            return hq_include

        def _hq_exclude() -> list:
            hq_exclude = []
            for state in search.criteria.exclusion.state:
                hq_exclude.append({"state": constants.STATES[state]})
            return hq_exclude

        api_filters = {
            "op": search.criteria.operator,
            "include": search.criteria.inclusion.keywords,
            "exclude": search.criteria.exclusion.keywords,
            "grata_employees_estimates_range": search.criteria.inclusion.employees_range,
            "ownership": search.criteria.inclusion.ownership,
            "headquarters": {
                "include": _hq_include(),
                "exclude": _hq_exclude(),
            },
        }
        # print(api_filters)
        return api_filters


class SourceScrubWrapper:
    def find_similar(domain: str, search: models.Search) -> dict:
        pass

    def find_by_criteria(search: models.Search) -> dict:
        pass

    def enrich(domain: str) -> dict:
        pass



from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass, field
from time import time

import pandas as pd
from dacite import from_dict

from gandai import query, models, gpt
from gandai.sources import GrataWrapper as grata
from gandai.sources import GoogleMapsWrapper as google


def enrich_company_by_domain(domain: str) -> None:
    company = query.find_company_by_domain(domain)
    if "company_uid" not in company.meta.keys():
        # company_uid is a grata specific field
        resp = grata.enrich(domain)
        # print(resp)
        # what happens when company is not found?
        # -> {'status': 404, 'errors': [{'field': 'domain', 'message': 'Cannot associate domain asdfasdf.com with a company.'}], 'linkedin': None, 'ownership': None}
        company.name = company.name or resp.get("name")
        company.description = resp.get("description")
        company.meta = {**company.meta, **resp}
        query.update_company(company)


def run_similarity_search(search: models.Search, domain: str) -> None:
    # dealcloud_companies = 
    grata_companies = grata.find_similar(domain=domain, search=search)
    query.insert_companies_as_targets(
        companies=grata_companies, search_uid=search.uid, actor_key="grata"
    )

def run_criteria_search(search: models.Search) -> None:
    grata_companies = grata.find_by_criteria(search)
    query.insert_companies_as_targets(
        companies=grata_companies, search_uid=search.uid, actor_key="grata"
    )

def run_maps_search(search: models.Search, event: models.Event) -> None:
    print("running maps search... may take 30++ seconds")
    start = time()
    top_n = event.data.get("top_n", 1)
    radius_miles = event.data.get("radius", 10)

    def process_area(area: str) -> None:
        centroids = gpt.get_top_zip_codes(area=area, top_n=top_n)
        print(f"searching {area} with {len(centroids)} centroids: {centroids}")

        place_ids = google.fetch_unique_place_ids(
            search_phrase=event.data["phrase"],
            locations=centroids,
            radius_miles=radius_miles,
        )
        print(f"{len(place_ids)} place_ids found in {area}")
        
        with ThreadPoolExecutor(max_workers=10) as executor:
            for place_id in place_ids:
                executor.submit(
                    google.build_target_from_place_id,
                    place_id=place_id,
                    search_uid=search.uid,
                    append_to_prompt=event.data['prompt']
                )

    # with ThreadPoolExecutor(max_workers=5) as executor:
    #     executor.map(process_area, e.data["areas"])
    for area in event.data["areas"]:
        process_area(area)


    print(f"🗺  Maps took {time() - start} seconds")

def process_event(event_id: int) -> None:
    event: models.Event = query.find_event_by_id(event_id)
    search = query.find_search(uid=event.search_uid)
    domain = event.domain
    if event.type == "create":
        # gpt enrich here
        pass
    elif event.type == "advance":
        enrich_company_by_domain(domain=domain) # 1 grata credit
    elif event.type == "validate":
        enrich_company_by_domain(domain=domain)
        run_similarity_search(search=search, domain=domain)
    elif event.type == "send":
        pass
    elif event.type == "client_approve":
        pass
    elif event.type == "reject":
        pass
    elif event.type == "client_reject":
        pass
    elif event.type == "conflict":
        enrich_company_by_domain(domain=domain)
        run_similarity_search(search=search, domain=domain)
    elif event.type == "client_conflict":
        enrich_company_by_domain(domain=domain)
        run_similarity_search(search=search, domain=domain)
    elif event.type == "criteria":
        if len(event.data['inclusion']["keywords"]) > 0:
            run_criteria_search(search=search)

    elif event.type == "maps":
        run_maps_search(search=search, event=event)

    elif event.type == "import":
        data = event.data
        query.insert_targets_from_domains(
            domains=data["domains"],
            search_uid=event.search_uid,
            actor_key=event.actor_key,
            stage=data.get("stage", "advance"),
        )

    elif event.type == "reset":
        print("💣 Resetting Inbox...")
        query.reset_inbox(search_uid=search.uid)

    


from dotenv import load_dotenv

load_dotenv()

from gandai import query, models, helpers
from gandai.db import connect_with_connector 

__all__ = ["query", "models", "helpers", "connect_with_connector"]



STATES = {
            "AL": "Alabama",
            "AK": "Alaska",
            "AZ": "Arizona",
            "AR": "Arkansas",
            "CA": "California",
            "CO": "Colorado",
            "CT": "Connecticut",
            "DE": "Delaware",
            "FL": "Florida",
            "GA": "Georgia",
            "HI": "Hawaii",
            "ID": "Idaho",
            "IL": "Illinois",
            "IN": "Indiana",
            "IA": "Iowa",
            "KS": "Kansas",
            "KY": "Kentucky",
            "LA": "Louisiana",
            "ME": "Maine",
            "MD": "Maryland",
            "MA": "Massachusetts",
            "MI": "Michigan",
            "MN": "Minnesota",
            "MS": "Mississippi",
            "MO": "Missouri",
            "MT": "Montana",
            "NE": "Nebraska",
            "NV": "Nevada",
            "NH": "New Hampshire",
            "NJ": "New Jersey",
            "NM": "New Mexico",
            "NY": "New York",
            "NC": "North Carolina",
            "ND": "North Dakota",
            "OH": "Ohio",
            "OK": "Oklahoma",
            "OR": "Oregon",
            "PA": "Pennsylvania",
            "RI": "Rhode Island",
            "SC": "South Carolina",
            "SD": "South Dakota",
            "TN": "Tennessee",
            "TX": "Texas",
            "UT": "Utah",
            "VT": "Vermont",
            "VA": "Virginia",
            "WA": "Washington",
            "WV": "West Virginia",
            "WI": "Wisconsin",
            "WY": "Wyoming",
        }

COUNTRIES = {
    "USA": "United States",
    "CAN": "Canada",
    "MEX": "Mexico",
    "GBR": "United Kingdom",
}


import openai
import plotly.express as px
import requests
from bs4 import BeautifulSoup

from gandai import models, query, secrets

openai.api_key = secrets.access_secret_version("OPENAI_KEY")


def ask_gpt(prompt: str, max_tokens: int = 60):
    response = openai.Completion.create(
        engine="text-davinci-003",  # or "text-curie-003" for a less expensive engine
        prompt=prompt,
        max_tokens=max_tokens,
    )

    return response.choices[0].text.strip()

def get_company_summary(domain: str) -> str:
    resp = requests.get(f"http://www.{domain}")
    soup = BeautifulSoup(resp.text, 'html.parser')
    homepage_text = soup.text.strip()

    PROMPT = f"""
    Summarize the products, services, and end markets of {domain}.

    Here is the homepage: {homepage_text}
    """

    resp = ask_gpt(PROMPT)
    return resp


def get_suggested_search_phrase(search: models.Search) -> str:
    # need up update this
    targets = query.search_targets(search_uid=search.uid).query()
    resp = ask_gpt(
        f"What search phrase would you use to search Google Maps to find companies that look like this: {targets['description'].tolist()}",
        max_tokens=60,
    )
    return resp


def get_top_zip_codes(area: str, top_n: int = 25) -> list:
    resp = ask_gpt(
        f"As a python array List[str], the top {top_n} zip codes in {area} are:",
        max_tokens=1000,
    )
    return eval(resp)[0:top_n] # top_n of 1 was giving me a ton of results


def summarize_search_by(targets):
    px.histogram(targets, x="employees", nbins=20, height=400, width=400).show()

    print(targets["ownership"].value_counts())

    GPT_PROMPT = f"""
    The top three geographic areas, based on the {[desc for desc in targets['description'].dropna().sample(20)]} are:
    """

    print(ask_gpt(prompt=GPT_PROMPT))

    GPT_PROMPT = f"""
    Give me the company type, products and services, and end markets {[desc for desc in targets['description'].dropna().sample(20)]} are:
    """

    print(ask_gpt(prompt=GPT_PROMPT, max_tokens=100))



from dataclasses import asdict, dataclass, field
from enum import Enum, auto
from typing import List, Optional


@dataclass
class Actor:
    # id: int = field(init=False)  # pk
    key: str  # phone number
    type: str
    name: str
    # created: int = field(init=False)
    # updated: int = field(init=False)


@dataclass(order=True)
class Company:
    domain: str  # unique
    name: Optional[str] = None
    description: Optional[str] = None
    meta: dict = field(default_factory=dict)
    id: int = field(default=None)  # primary key
    uid: Optional[int] = field(default=None)  # foreign key


class EventType(str, Enum):
    CREATE = auto()
    ADVANCE = auto()
    VALIDATE = auto()
    SEND = auto()
    CLIENT_APPROVE = auto()
    CONFLICT = auto()
    REJECT = auto()
    CLIENT_REJECT = auto()
    CRITERIA = auto()


@dataclass
class Event:
    search_uid: int  # fk # add index
    domain: Optional[str]  # fk
    actor_key: str  # fk
    type: str  # build, advance, qualify, reject, conflict, rate
    data: dict = field(default_factory=dict)
    id: int = field(default=None)  # pk
    # created: int = field(init=False)


@dataclass
class Comment(Event):
    def __post_init__(self):
        # self.type = EventType.COMMENT
        assert isinstance(self.data["comment"], str)


@dataclass
class Rating(Event):
    def __post_init__(self):
        assert isinstance(self.data["rating"], int)


# @dataclass
# class Checkpoint:
#     # actor_key: str  # foreign key
#     event_id: int  # foreign key
#     id: int = field(default=None)  # primary key
# created: int  # timestamp


@dataclass
class Inclusion:
    keywords: List[str] = field(default_factory=list)
    employees_range: List[int] = field(default_factory=lambda: [100, 500])
    ownership: List[str] = field(default_factory=lambda: ["bootstrapped"])
    country: List[str] = field(default_factory=lambda: ["USA"])
    state: List[str] = field(default_factory=list)
    city: List[str] = field(default_factory=list)


@dataclass
class Exclusion:
    keywords: List[str] = field(default_factory=list)
    state: List[str] = field(default_factory=list)


@dataclass
class Criteria:
    inclusion: Inclusion = field(default_factory=Inclusion)
    exclusion: Exclusion = field(default_factory=Exclusion)
    result_count: int = 25
    operator: str = "any"


@dataclass
class Maps:
    phrase: str = ""
    areas: List[str] = field(default_factory=list)
    top_n: int = 1
    radius: int = 25
    prompt: str = ""


@dataclass
class Search:
    uid: int  # foreign key, dealcloud id
    label: str  # maps to dealcloud engagement name
    meta: dict = field(default_factory=dict)
    criteria: Criteria = field(default_factory=Criteria)
    maps: Maps = field(default_factory=Maps)
    fields: List[str] = field(default_factory=list)

    def __post_init__(self):
        if self.fields == []:
            self.fields = [
                "name",
                "domain",
                "year_founded",
                "headquarters",
                "description",
                "rating",
                "employees",
                "linkedin",
                "products",
                "services",
                "end_markets",
                "contacts",
                "comments",
            ]



import os
import random
from dataclasses import asdict, dataclass, field
from hashlib import md5
from time import time

import pandas as pd
from twilio.rest import Client

from gandai.secrets import access_secret_version

# from gandai.models.User import user_exists # create_user

ds = {}
twilio_client = Client(
    access_secret_version("TWILIO_APP"), access_secret_version("TWILIO_TOKEN")
)


@dataclass
class Auth:
    key: str = field(init=False)
    phone: str
    code: str  # 6 digit
    expires: int = field(init=False)
    token: str = field(init=False)

    def __post_init__(self):
        assert len(str(self.code)) == 6
        assert len(str(self.phone)) == 10
        SEVEN_DAYS = 7 * 86400
        self.expires = int(time()) + SEVEN_DAYS
        self.key = f"auth/{self.phone}/{self.expires}"
        self.token = md5(self.key.encode()).hexdigest()


def _send_code(auth: Auth) -> None:
    message = twilio_client.messages.create(
        to=auth.phone,
        from_=access_secret_version("TWILIO_NUMBER"),
        body=f"{auth.code} is your TargetSelect authentication code.",
    )
    print(f"Login Sent to {auth.phone}")


def send_code(phone: str) -> None:
    # Event.create(actor_key=phone)
    auth_code = str(random.randint(100000, 999999))
    auth = Auth(phone=phone, code=auth_code)
    ds[auth.key] = asdict(auth)
    _send_code(auth)


def authenticate(code: str) -> Auth:
    code = code.strip()
    keys = ds.keys()
    df = pd.DataFrame([ds[k] for k in keys])
    df = df[df["expires"] > int(time())]
    df = df[df["code"] == code]
    if len(df) > 0:
        # could parse to Auth and back to dict
        return df.to_dict(orient="records")[0]
    else:
        return None


def validate(token: str) -> bool:
    pass



import os
from google.cloud import tasks_v2
from threading import Thread

from gandai import query, main

import json

from time import sleep


def process_event_local(event_id: int):
    # Simulate the processing time
    event = query.find_event_by_id(event_id=event_id)
    main.process_event(event_id=event.id)
    print(f"Processing event {event_id}")

def process_event_task(event_id: int) -> None:
    print(os.getenv("ENV"))
    if os.getenv("ENV") == "local":
        # this may be hard to debug
        process_event_local(event_id=event_id)
        # Thread(target=process_event_local, args=(event_id,)).start()
    else:
        print("omglol")
        client = tasks_v2.CloudTasksClient()
        project = os.getenv("PROJECT_ID", "targetselect-staging")
        queue = "phq"
        location = "us-central1"
        parent = client.queue_path(project, location, queue)
        task = {
            "app_engine_http_request": {
                "http_method": tasks_v2.HttpMethod.POST,
                "app_engine_routing": {"service": "api"},
                "relative_uri": "/process_event",
                "headers": {"Content-type": "application/json"},
                "body": json.dumps({"event_id": event_id}).encode(),
            }
        }
        response = client.create_task(parent=parent, task=task)
        return response
